import os
import numpy as np
from hysop.constants import HYSOP_REAL, ORDER


def random_init(shape, mpi_comm):
    # Create a folder to store all random arrays
    d = 'rand_init'
    if mpi_comm.Get_rank() == 0:
        if not os.path.exists(d):
            os.makedirs(d)
    mpi_comm.Barrier()
    file_name = "{0}_{1}_{2}".format(*shape)
    file_name += "_{0}p_{1}.dat".format(mpi_comm.Get_size(),
                                        mpi_comm.Get_rank())
    try:
        randX = np.asarray(
            np.reshape(np.fromfile(os.path.join(d, 'randX_' + file_name),
                                   dtype=HYSOP_REAL), shape),
            dtype=HYSOP_REAL, order=ORDER)
        randY = np.asarray(
            np.reshape(np.fromfile(os.path.join(d, 'randY_' + file_name),
                                   dtype=HYSOP_REAL), shape),
            dtype=HYSOP_REAL, order=ORDER)
        randZ = np.asarray(
            np.reshape(np.fromfile(os.path.join(d, 'randZ_' + file_name),
                                   dtype=HYSOP_REAL), shape),
            dtype=HYSOP_REAL, order=ORDER)
    except IOError:
        randX = np.asarray(np.random.random(shape),
                           dtype=HYSOP_REAL, order=ORDER) - 0.5
        randY = np.asarray(np.random.random(shape),
                           dtype=HYSOP_REAL, order=ORDER) - 0.5
        randZ = np.asarray(np.random.random(shape),
                           dtype=HYSOP_REAL, order=ORDER) - 0.5
        randX.tofile(os.path.join(d, 'randX_' + file_name))
        randY.tofile(os.path.join(d, 'randY_' + file_name))
        randZ.tofile(os.path.join(d, 'randZ_' + file_name))
    return randX, randY, randZ
