
%module fft

// -*- C++ -*-
%{
  #include "fftw3.h"
  #include "fft/fftw3.h"
  #include "fft/extension.h"
  #include "fft/transform.h"
  #include "fft/fftDomainConfiguration.h"
%}

/* fftw3.h */
#if !defined(FFTW_HAS_FFTW3F_THREADS) || !defined(FFTW_HAS_FFTW3F_OMP)
    %ignore fftwf_init_threads();
    %ignore fftwf_plan_with_nthreads(int);
    %ignore fftwf_cleanup_threads();
#endif

#if !defined(FFTW_HAS_FFTW3D_THREADS) || !defined(FFTW_HAS_FFTW3D_OMP)
    %ignore fftw_init_threads();
    %ignore fftw_plan_with_nthreads(int);
    %ignore fftw_cleanup_threads();
#endif

#if !defined(FFTW_HAS_FFTW3L_THREADS) || !defined(FFTW_HAS_FFTW3L_OMP)
    %ignore fftwl_init_threads();
    %ignore fftwl_plan_with_nthreads(int);
    %ignore fftwl_cleanup_threads();
#endif

#ifdef HAS_QUADMATHS
    #if !defined(FFTW_HAS_FFTW3Q_THREADS) || !defined(FFTW_HAS_FFTW3Q_OMP)
        %ignore fftwq_init_threads();
        %ignore fftwq_plan_with_nthreads(int);
        %ignore fftwq_cleanup_threads();
    #endif
#endif
%rename(is_) fftw_iodim_do_not_use_me::is;
%rename(is_) fftw_iodim64_do_not_use_me::is;
%include "fftw3.h"


/* fftw3 c++ wrappers */
%include "fft/fftw3.h"
%template(Fftw3f) hysop::fft::Fftw3<float>;
%template(Fftw3d) hysop::fft::Fftw3<double>;
%template(Fftw3l) hysop::fft::Fftw3<long double>;
%template(Fftw3q) hysop::fft::Fftw3<__float128>;


/* fft transforms */
%ignore hysop::fft::operator<<;
%include "fft/transform.h"
%extend hysop::fft::Transform {
    const char* __str__() const { 
        return $self->toString().c_str(); 
    }
}


/* fft extensions */
%ignore hysop::fft::operator<<;
%include "fft/extension.h"


/* fft domain configuration */
%ignore hysop::fft::operator<<;
%ignore hysop::fft::FftDomainConfiguration::operator[];
%include "fft/fftDomainConfiguration.h"

%define INSTANTIATE_FFT_DOMAIN_CONFIG_IMPL(CLASS_NAME,CLASS_TEMPLATE...) 
    %template(CLASS_NAME) CLASS_TEMPLATE; 
    %extend CLASS_TEMPLATE {
        CLASS_TEMPLATE::ExtensionPair __get_item__(unsigned int k) const {
            return $self->operator[](k);
        }
        const char* __str__() const {
            std::stringstream ss;
            ss << *($self);
            return ss.str().c_str();
        }
    }
%enddef

%define INSTANTIATE_FFT_DOMAIN_CONFIG(INT) 
    INSTANTIATE_FFT_DOMAIN_CONFIG_IMPL(FftDomainConfiguration ## INT ## D, hysop::fft::FftDomainConfiguration<INT>)
%enddef

%formacro(INSTANTIATE_FFT_DOMAIN_CONFIG, INSTANTIATED_DIMENSIONS)

